'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _v = require('uuid/v4');

var _v2 = _interopRequireDefault(_v);

var _action_types = require('./action_types');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var merge = function merge(viewModelName, aggregateId, state) {
  return {
    type: _action_types.MERGE,
    viewModelName: viewModelName,
    aggregateId: aggregateId,
    state: state
  };
};

var sendCommand = function sendCommand(_ref) {
  var command = _ref.command,
      aggregateId = _ref.aggregateId,
      aggregateName = _ref.aggregateName,
      payload = _ref.payload;
  return {
    type: _action_types.SEND_COMMAND,
    command: command,
    aggregateId: aggregateId,
    aggregateName: aggregateName,
    payload: payload
  };
};

var subscribeViewModel = function subscribeViewModel(viewModelName, aggregateId) {
  return {
    type: _action_types.SUBSCRIBE_VIEWMODEL,
    viewModelName: viewModelName,
    aggregateId: aggregateId
  };
};

var unsubscribeViewModel = function unsubscribeViewModel(viewModelName, aggregateId) {
  return {
    type: _action_types.UNSUBSCRIBE_VIEWMODEL,
    viewModelName: viewModelName,
    aggregateId: aggregateId
  };
};

var subscribeReadModel = function subscribeReadModel(readModelName, resolverName, parameters, isReactive) {
  return {
    type: _action_types.SUBSCRIBE_READMODEL,
    readModelName: readModelName,
    resolverName: resolverName,
    parameters: parameters,
    isReactive: isReactive
  };
};

var unsubscribeReadModel = function unsubscribeReadModel(readModelName, resolverName) {
  return {
    type: _action_types.UNSUBSCRIBE_READMODEL,
    readModelName: readModelName,
    resolverName: resolverName
  };
};

var loadReadModelInitialState = function loadReadModelInitialState(readModelName, resolverName, initialState, serialId) {
  return {
    type: _action_types.READMODEL_LOAD_INITIAL_STATE,
    readModelName: readModelName,
    resolverName: resolverName,
    initialState: initialState,
    serialId: serialId
  };
};

var dropReadModelState = function dropReadModelState(readModelName, resolverName) {
  return {
    type: _action_types.READMODEL_DROP_STATE,
    readModelName: readModelName,
    resolverName: resolverName
  };
};

var provideViewModels = function provideViewModels(viewModels) {
  return {
    type: _action_types.PROVIDE_VIEW_MODELS,
    viewModels: viewModels
  };
};

var disconnect = function disconnect(reason) {
  return {
    type: _action_types.DISCONNECT,
    reason: reason
  };
};

var hotModuleReplacement = function hotModuleReplacement() {
  return {
    type: _action_types.HOT_MODULE_REPLACEMENT,
    hotModuleReplacementId: (0, _v2.default)()
  };
};

exports.default = {
  merge: merge,
  sendCommand: sendCommand,
  subscribeViewModel: subscribeViewModel,
  unsubscribeViewModel: unsubscribeViewModel,
  provideViewModels: provideViewModels,
  subscribeReadModel: subscribeReadModel,
  unsubscribeReadModel: unsubscribeReadModel,
  loadReadModelInitialState: loadReadModelInitialState,
  dropReadModelState: dropReadModelState,
  disconnect: disconnect,
  hotModuleReplacement: hotModuleReplacement
};
module.exports = exports['default'];