'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _extends8 = require('babel-runtime/helpers/extends');

var _extends9 = _interopRequireDefault(_extends8);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _getPrototypeOf = require('babel-runtime/core-js/object/get-prototype-of');

var _getPrototypeOf2 = _interopRequireDefault(_getPrototypeOf);

var _create = require('babel-runtime/core-js/object/create');

var _create2 = _interopRequireDefault(_create);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _defineProperty4 = require('babel-runtime/core-js/object/define-property');

var _defineProperty5 = _interopRequireDefault(_defineProperty4);

var _getOwnPropertyDescriptor = require('babel-runtime/core-js/object/get-own-property-descriptor');

var _getOwnPropertyDescriptor2 = _interopRequireDefault(_getOwnPropertyDescriptor);

var _getOwnPropertyNames = require('babel-runtime/core-js/object/get-own-property-names');

var _getOwnPropertyNames2 = _interopRequireDefault(_getOwnPropertyNames);

exports.default = createReadModelsReducer;

var _action_types = require('./action_types');

var _diffJson = require('diff-json');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function copyResolveSerials(oldState, newState) {
  (0, _getOwnPropertyNames2.default)(oldState).forEach(function (key) {
    var propertyDescriptor = (0, _getOwnPropertyDescriptor2.default)(oldState, key);

    if (key.indexOf('resolve-serial:') !== 0 || propertyDescriptor.enumerable || propertyDescriptor.writable) {
      return;
    }

    (0, _defineProperty5.default)(newState, key, propertyDescriptor);
  });
}

function refreshUpdatedObjects(updatedObject, changes) {
  var embededKey = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '$index';

  var _loop = function _loop(_ref) {
    var key = _ref.key,
        nextChanges = _ref.changes,
        nextEmbededKey = _ref.embededKey;

    var calcKey = embededKey !== '$index' && Array.isArray(updatedObject) ? updatedObject.reduce(function (result, value, idx) {
      return value[embededKey] === key ? idx : result;
    }, 0) : key;

    if (updatedObject[calcKey] == null || updatedObject[calcKey].constructor === String || updatedObject[calcKey].constructor === Number || updatedObject[calcKey].constructor === Date) {
      return 'continue';
    }

    if (Array.isArray(updatedObject[calcKey])) {
      updatedObject[calcKey] = [].concat((0, _toConsumableArray3.default)(updatedObject[calcKey]));
    } else {
      var nextObject = (0, _create2.default)((0, _getPrototypeOf2.default)(updatedObject[calcKey]));
      (0, _assign2.default)(nextObject, updatedObject[calcKey]);
      updatedObject[calcKey] = nextObject;
    }

    if (Array.isArray(nextChanges)) {
      refreshUpdatedObjects(updatedObject[calcKey], nextChanges, nextEmbededKey);
    }
  };

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator3.default)(changes), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ref = _step.value;

      var _ret = _loop(_ref);

      if (_ret === 'continue') continue;
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }
}

function createReadModelsReducer() {
  return function () {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var action = arguments[1];

    switch (action.type) {
      case _action_types.READMODEL_LOAD_INITIAL_STATE:
        {
          var readModelName = action.readModelName,
              resolverName = action.resolverName,
              initialState = action.initialState,
              serialId = action.serialId;

          var nextState = (0, _extends9.default)({}, state, (0, _defineProperty3.default)({}, readModelName, (0, _extends9.default)({}, state[readModelName] || {}, (0, _defineProperty3.default)({}, resolverName, initialState))));

          copyResolveSerials(state, nextState);
          Object.defineProperty(nextState, 'resolve-serial:' + readModelName + ':' + resolverName, {
            configurable: true,
            writable: false,
            enumerable: false,
            value: serialId
          });

          return nextState;
        }

      case _action_types.READMODEL_DROP_STATE:
        {
          var _readModelName = action.readModelName,
              _resolverName = action.resolverName;

          var _nextState = (0, _extends9.default)({}, state, (0, _defineProperty3.default)({}, _readModelName, (0, _extends9.default)({}, state[_readModelName] || {}, (0, _defineProperty3.default)({}, _resolverName, null))));

          copyResolveSerials(state, _nextState);
          delete _nextState['resolve-serial:' + _readModelName + ':' + _resolverName];
          delete _nextState[_readModelName][_resolverName];

          return _nextState;
        }

      case _action_types.READMODEL_SUBSCRIPTION_DIFF:
        {
          var _readModelName2 = action.readModelName,
              _resolverName2 = action.resolverName,
              _serialId = action.serialId,
              diff = action.diff;


          if (!state[_readModelName2] || !state[_readModelName2].hasOwnProperty(_resolverName2) || state['resolve-serial:' + _readModelName2 + ':' + _resolverName2] !== _serialId) {
            return state;
          }

          var wrappedResolverState = {
            wrap: state[_readModelName2][_resolverName2]
          };

          (0, _diffJson.applyChanges)(wrappedResolverState, diff);

          refreshUpdatedObjects(wrappedResolverState, diff);

          var _nextState2 = (0, _extends9.default)({}, state, (0, _defineProperty3.default)({}, _readModelName2, (0, _extends9.default)({}, state[_readModelName2], (0, _defineProperty3.default)({}, _resolverName2, wrappedResolverState.wrap))));

          copyResolveSerials(state, _nextState2);

          return _nextState2;
        }

      default:
        return state;
    }
  };
}
module.exports = exports['default'];