'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

exports.createResolveMiddleware = createResolveMiddleware;

var _action_types = require('./action_types');

var _utils = require('./utils');

var _actions = require('./actions');

var _actions2 = _interopRequireDefault(_actions);

var _subscribe_adapter = require('./subscribe_adapter');

var _subscribe_adapter2 = _interopRequireDefault(_subscribe_adapter);

var _send_command = require('./send_command');

var _send_command2 = _interopRequireDefault(_send_command);

var _create_actions = require('./create_actions');

var _create_actions2 = _interopRequireDefault(_create_actions);

var _mock_subscribe_adapter = require('./mock_subscribe_adapter');

var _mock_subscribe_adapter2 = _interopRequireDefault(_mock_subscribe_adapter);

var _subscribe_view_model = require('./subscribe_view_model');

var _subscribe_view_model2 = _interopRequireDefault(_subscribe_view_model);

var _unsubscribe_view_model = require('./unsubscribe_view_model');

var _unsubscribe_view_model2 = _interopRequireDefault(_unsubscribe_view_model);

var _subscribe_read_model = require('./subscribe_read_model');

var _subscribe_read_model2 = _interopRequireDefault(_subscribe_read_model);

var _unsubscribe_read_model = require('./unsubscribe_read_model');

var _unsubscribe_read_model2 = _interopRequireDefault(_unsubscribe_read_model);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var REFRESH_TIMEOUT = 1000;

var isClient = typeof window !== 'undefined';

function createResolveMiddleware(_ref) {
  var _ref$viewModels = _ref.viewModels,
      viewModels = _ref$viewModels === undefined ? [] : _ref$viewModels,
      _ref$readModels = _ref.readModels,
      readModels = _ref$readModels === undefined ? [] : _ref$readModels,
      _ref$aggregates = _ref.aggregates,
      aggregates = _ref$aggregates === undefined ? [] : _ref$aggregates,
      _ref$subscribeAdapter = _ref.subscribeAdapter,
      subscribeAdapter = _ref$subscribeAdapter === undefined ? _subscribe_adapter2.default : _ref$subscribeAdapter,
      origin = _ref.origin,
      rootPath = _ref.rootPath;

  var subscribers = {
    viewModels: {},
    aggregateIds: {}
  };

  var requests = {};
  var loading = {};
  var readModelSubscriptions = {};

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator3.default)(viewModels), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ref2 = _step.value;
      var name = _ref2.name;

      loading[name] = {};
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  var aggregateActions = {};
  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = (0, _getIterator3.default)(aggregates), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var aggregate = _step2.value;

      (0, _assign2.default)(aggregateActions, (0, _create_actions2.default)(aggregate));
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  var orderedFetch = (0, _utils.createOrderedFetch)();

  return function (store) {
    var adapter = (isClient ? subscribeAdapter : _mock_subscribe_adapter2.default)({
      origin: origin,
      rootPath: rootPath
    });
    adapter.onEvent(function (event) {
      return store.dispatch(event);
    });
    adapter.onDisconnect(function (error) {
      store.dispatch(_actions2.default.disconnect(error));

      (0, _keys2.default)(readModelSubscriptions).forEach(function (subscriptionKey) {
        readModelSubscriptions[subscriptionKey].refresh();
      });
    });

    var context = {
      origin: origin,
      rootPath: rootPath,
      adapter: adapter,
      viewModels: viewModels,
      readModels: readModels,
      aggregates: aggregates,
      aggregateActions: aggregateActions,
      store: store,
      subscribers: subscribers,
      requests: requests,
      readModelSubscriptions: readModelSubscriptions,
      orderedFetch: orderedFetch
    };

    Object.defineProperty(store.getState, 'isLoadingViewModel', {
      enumerable: false,
      configurable: false,
      writable: false,
      value: function value(viewModelName, aggregateId) {
        return !!loading[viewModelName][aggregateId];
      }
    });
    Object.defineProperty(store.getState, 'aggregateActions', {
      enumerable: false,
      configurable: false,
      writable: false,
      value: aggregateActions
    });

    store.dispatch(_actions2.default.provideViewModels(viewModels));

    if (!isClient) {
      return function (next) {
        return function (action) {
          return next(action);
        };
      };
    }

    var hotModuleReplacementId = void 0;

    return function (next) {
      return function (action) {
        switch (action.type) {
          case _action_types.HOT_MODULE_REPLACEMENT:
            {
              if (!hotModuleReplacementId) {
                hotModuleReplacementId = action.hotModuleReplacementId;
              }
              if (hotModuleReplacementId !== action.hotModuleReplacementId) {
                window.location.reload();
              }
              break;
            }
          case _action_types.SUBSCRIBE_VIEWMODEL:
            {
              var viewModelName = action.viewModelName,
                  aggregateId = action.aggregateId;


              loading[viewModelName][aggregateId] = true;

              (0, _subscribe_view_model2.default)(context, action).catch(function (error) {
                return setTimeout(function () {
                  // eslint-disable-next-line no-console
                  console.error(error);
                  store.dispatch(action);
                }, REFRESH_TIMEOUT);
              });
              break;
            }
          case _action_types.UNSUBSCRIBE_VIEWMODEL:
            {
              var _viewModelName = action.viewModelName,
                  _aggregateId = action.aggregateId;


              delete loading[_viewModelName][_aggregateId];

              (0, _unsubscribe_view_model2.default)(context, action);
              break;
            }
          case _action_types.SUBSCRIBE_READMODEL:
            {
              (0, _subscribe_read_model2.default)(context, action);
              break;
            }
          case _action_types.UNSUBSCRIBE_READMODEL:
            {
              (0, _unsubscribe_read_model2.default)(context, action);
              break;
            }
          case _action_types.SEND_COMMAND:
            {
              (0, _send_command2.default)(context, action);
              break;
            }
          default:
        }

        return next(action);
      };
    };
  };
}

exports.default = createResolveMiddleware;