'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _utils = require('./utils');

var _actions = require('./actions');

var _actions2 = _interopRequireDefault(_actions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var subscribeReadModel = function subscribeReadModel(_ref, action) {
  var origin = _ref.origin,
      rootPath = _ref.rootPath,
      store = _ref.store,
      readModelSubscriptions = _ref.readModelSubscriptions,
      adapter = _ref.adapter,
      orderedFetch = _ref.orderedFetch;
  var readModelName = action.readModelName,
      resolverName = action.resolverName,
      parameters = action.parameters,
      isReactive = action.isReactive;


  var subscriptionKey = readModelName + ':' + resolverName;
  if (readModelSubscriptions.hasOwnProperty(subscriptionKey)) {
    return;
  }

  var fetchReadModel = function fetchReadModel() {
    if (!readModelSubscriptions.hasOwnProperty(subscriptionKey)) {
      return;
    }
    var checkSelfPromise = function checkSelfPromise(selfPromise) {
      return readModelSubscriptions.hasOwnProperty(subscriptionKey) && selfPromise === readModelSubscriptions[subscriptionKey].promise;
    };

    var selfPromise = _promise2.default.resolve().then((0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
      var socketId, response, _ref3, result, timeToLive, serialId;

      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.prev = 0;

              if (checkSelfPromise(selfPromise)) {
                _context.next = 3;
                break;
              }

              return _context.abrupt('return');

            case 3:
              _context.next = 5;
              return adapter.getClientId();

            case 5:
              socketId = _context.sent;

              readModelSubscriptions[subscriptionKey].socketId = socketId;

              if (checkSelfPromise(selfPromise)) {
                _context.next = 9;
                break;
              }

              return _context.abrupt('return');

            case 9:
              _context.next = 11;
              return orderedFetch((0, _utils.getRootableUrl)(origin, rootPath, '/api/query/' + readModelName + '/' + resolverName), {
                method: 'POST',
                headers: { 'Content-Type': 'application/json' },
                credentials: 'same-origin',
                body: (0, _stringify2.default)({
                  isReactive: isReactive,
                  socketId: socketId,
                  parameters: parameters
                })
              });

            case 11:
              response = _context.sent;

              if (response.ok) {
                _context.next = 14;
                break;
              }

              throw new Error();

            case 14:
              _context.next = 16;
              return response.json();

            case 16:
              _ref3 = _context.sent;
              result = _ref3.result;
              timeToLive = _ref3.timeToLive;
              serialId = _ref3.serialId;

              if (checkSelfPromise(selfPromise)) {
                _context.next = 22;
                break;
              }

              return _context.abrupt('return');

            case 22:
              store.dispatch(_actions2.default.loadReadModelInitialState(readModelName, resolverName, result, serialId));

              if (isReactive) {
                _context.next = 25;
                break;
              }

              return _context.abrupt('return');

            case 25:
              _context.next = 27;
              return (0, _utils.delay)(timeToLive);

            case 27:
              fetchReadModel();
              _context.next = 35;
              break;

            case 30:
              _context.prev = 30;
              _context.t0 = _context['catch'](0);
              _context.next = 34;
              return (0, _utils.delay)(1000);

            case 34:
              fetchReadModel();

            case 35:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, undefined, [[0, 30]]);
    })));

    readModelSubscriptions[subscriptionKey].promise = selfPromise;
  };

  readModelSubscriptions[subscriptionKey] = {
    promise: _promise2.default.resolve(),
    refresh: fetchReadModel,
    socketId: null
  };

  fetchReadModel();
};

exports.default = subscribeReadModel;
module.exports = exports['default'];